# -*- coding: utf-8 -*-
# Copyright 2015 Tencent
# Author: Joe Lei <joelei@tencent.com>
import logging
import sys

from autowatch import cmd

LOG = logging.getLogger(__name__)


if __name__ == "__main__":
    if len(sys.argv) < 2:
        print "tricks_file is need."
        sys.exit(1)
    tricks_file = sys.argv[1]
    cmd.auto(tricks_file)



# -*- coding: utf-8 -*-
# Copyright 2015 Tencent
# Author: Joe Lei <joelei@tencent.com>
import logging
import logging.config
import os.path
import time

from watchdog.observers import Observer

from autowatch import settings
from autowatch.events import AutoWatchYaml

logging.config.dictConfig(settings.LOGGING)
LOG = logging.getLogger(__name__)


def auto(tricks_file):
    LOG.info(tricks_file)
    if not os.path.exists(tricks_file):
        raise IOError("cannot find tricks file: %s" % tricks_file)
    tricks_file = os.path.abspath(tricks_file)
    dir_path = os.path.dirname(tricks_file)
    LOG.info(dir_path)
    event_handler = AutoWatchYaml(tricks_file)
    observer = Observer()
    observer.schedule(event_handler, dir_path, recursive=False)
    observer.start()
    LOG.info("Start AutoWatchYaml Success")
    try:
        while True:
            time.sleep(1)
    except KeyboardInterrupt:
        observer.stop()
    observer.join()
    LOG.error("AutoWatchYaml Stoped")



# -*- coding: utf-8 -*-
# Copyright 2015 Tencent
# Author: Joe Lei <joelei@tencent.com>
import os.path

ROOT_PACKAGE = __name__.split('.')[0]
PATH = os.path.abspath('.')

LOGGING = {
    'version': 1,
    'disable_existing_loggers': False,
    'formatters': {
        'simple': {
            'format': '%(asctime)s [%(levelname)s] %(name)s %(message)s'
        },
    },
    'handlers': {
        'console': {
            'level': 'DEBUG',
            'class': 'logging.StreamHandler',
            'formatter': 'simple'
        },
        'file': {
            'level': 'DEBUG',
            'class': 'logging.FileHandler',
            'filename': os.path.join(PATH, 'autowatch.log'),
            'formatter': 'simple'
        }
    },
    'loggers': {
        ROOT_PACKAGE: {
            'handlers': ['file'],
            'level': 'DEBUG',
            'propagate': False,
        },
        '__main__': {
            'handlers': ['file'],
            'level': 'DEBUG',
            'propagate': False,
        },
    }
}



# -*- coding: utf-8 -*-
# Copyright 2015 Tencent
# Author: Joe Lei <joelei@tencent.com>
import abc
import threading
import logging
import os

# from watchdog.utils import echo
from watchdog.observers import Observer
from watchdog.events import PatternMatchingEventHandler
from watchdog.watchmedo import (add_to_sys_path, load_config,
                                schedule_tricks, CONFIG_KEY_TRICKS,
                                CONFIG_KEY_PYTHON_PATH)

LOG = logging.getLogger(__name__)


class AutoWatchHandler(PatternMatchingEventHandler):

    __metaclass__ = abc.ABCMeta

    def __init__(self, *args, **kwargs):
        super(AutoWatchHandler, self).__init__(*args, **kwargs)
        self.event = threading.Event()

    def start(self):
        w = self.watcher()
        w.start()

    def stop(self):
        self.event.set()

    # @echo.echo
    def on_any_event(self, event):
        self.stop()
        self.start()

    @abc.abstractmethod
    def watcher(self):
        """watcher
        """


class AutoWatchYaml(AutoWatchHandler):
    def __init__(self, files):
        patterns = ['*' + os.path.basename(files)]
        self.files = files
        LOG.info("PatternMatching: %s", patterns)
        super(AutoWatchYaml, self).__init__(patterns=patterns)

    def watcher(self):
        w = YamlTrick(self.event, [self.files])
        return w


class YamlTrick(threading.Thread):
    def __init__(self, event, files):
        super(YamlTrick, self).__init__()
        self.event = event
        self.event.clear()
        self.files = files

    def run(self):
        observers = []
        LOG.info("Start %s" % str(self))
        LOG.info('YamlTrick: %s', self.files)
        for tricks_file in self.files:
            observer = Observer(timeout=1)

            if not os.path.exists(tricks_file):
                raise IOError("cannot find tricks file: %s" % tricks_file)

            config = load_config(tricks_file)

            try:
                tricks = config[CONFIG_KEY_TRICKS]
            except KeyError:
                raise KeyError("No `%s' key specified in %s." % (
                               CONFIG_KEY_TRICKS, tricks_file))

            if CONFIG_KEY_PYTHON_PATH in config:
                add_to_sys_path(config[CONFIG_KEY_PYTHON_PATH])

            dir_path = os.path.dirname(tricks_file)
            if not dir_path:
                dir_path = os.path.relpath(os.getcwd())
            schedule_tricks(observer, tricks, dir_path, True)
            observer.start()
            observers.append(observer)

        self.event.wait()
        for o in observers:
            o.unschedule_all()
            o.stop()
        for o in observers:
            o.join()
        LOG.error("Stoped %s", str(self))



# -*- coding: utf-8 -*-
# Copyright 2014 Powerleader, PLCLOUD
# Author: Joe Lei <jiaomin.lei@powerleader.com.cn>
import subprocess
import os
import logging

from watchdog.tricks import Trick
from watchdog.utils import has_attribute

LOG = logging.getLogger(__name__)


class Rsync(Trick):
    """Rsync Trick
    """

    def __init__(self, shell_command=None, patterns=None, ignore_patterns=None,
                 ignore_directories=False, wait_for_process=False,
                 drop_during_process=True, **kwargs):
        super(Rsync, self).__init__(patterns, ignore_patterns, ignore_directories)
        self.shell_command = shell_command
        if not self.shell_command:
            self.shell_command = 'rsync -avz ${source_directory} %s' % kwargs.get('target')
        self.wait_for_process = wait_for_process
        self.drop_during_process = drop_during_process
        self.process = None
        self.kwargs = kwargs
        self.source_directory = kwargs.get('source_directory')
        os.environ['RSYNC_PASSWORD'] = 'joelei123'

    def on_any_event(self, event):
        from string import Template

        if self.drop_during_process and self.process and self.process.poll() is None:
            return

        if event.is_directory:
            object_type = 'directory'
        else:
            object_type = 'file'

        context = {
            'watch_src_path': event.src_path,
            'watch_dest_path': '',
            'watch_event_type': event.event_type,
            'watch_object': object_type,
        }

        if self.shell_command is None:
            if has_attribute(event, 'dest_path'):
                context.update({'dest_path': event.dest_path})
                command = 'echo "${watch_event_type} ${watch_object} from ${watch_src_path} to ${watch_dest_path}"'
            else:
                command = 'echo "${watch_event_type} ${watch_object} ${watch_src_path}"'
        else:
            if has_attribute(event, 'dest_path'):
                context.update({'watch_dest_path': event.dest_path})
            command = self.shell_command
        context.update(self.kwargs)
        driver, path = self.source_directory.split(':')
        context['source_directory'] = '/cygdrive/%s%s' % (driver.lower(), path.replace('\\', '/'))

        command = Template(command).safe_substitute(**context)
        if self.ignore_directories:
            for i in self.ignore_directories:
                command += ' --exclude %s ' % i

        LOG.info(">>> %s" % command)
        self.process = subprocess.Popen(command, shell=True)
        if self.wait_for_process:
            self.process.wait()



# -*- coding: utf-8 -*-
# Copyright 2015 Tencent
# Author: Joe Lei <thezero12@hotmail.com>
import setuptools
import sys

if sys.platform.startswith("win"):
    scripts = ['bin/autowatchme.pyw']
else:
    scripts = []

setuptools.setup(
    scripts=scripts,
    setup_requires=['pbr'],
    pbr=True
)
